@@ -1,9 +1,13 @@
 
-lagData <- function(data, lags, consec = NULL) {
+lagData <- function(data, 
+                    lags, 
+                    consec = NULL) {
 
 
   # ---------- Compute Aux Variables ----------
 
+  data <- as.matrix(data) # turn into matrix
+  
   max_lag <- max(lags) # maximum lag
   lags_ext <- 1:max(lags) # makes it easier to delete right columns
 
@@ -14,10 +18,10 @@ lagData <- function(data, lags, consec = NULL) {
 
   data_response <- data[-c(1:max_lag), ]
 
-  if(!is.null(consec)) m_consec <- matrix(NA, nrow = n_var, ncol = n_lags)
-
   # browser()
 
+  if(!is.null(consec)) m_consec <- matrix(NA, nrow = n_var, ncol = n_lags)
+
   # ---------- Lag Variables ----------
 
   lag_pos <- 1 # to make sure that the list is filled successively, if not a full sequence (e.g. lags = c(1,5)); otherwise this leads to problems later in the code
@@ -31,11 +35,13 @@ lagData <- function(data, lags, consec = NULL) {
     if(lag == max_lag) {
 
       l_data_lags[[lag_pos]] <- data[-((n-end+1):n), ]
+      l_data_lags[[lag_pos]] <- matrix(l_data_lags[[lag_pos]], ncol = p, nrow = n_var) # enforce 2 dimensions to ensure that the function also works for 1 variable only
       colnames(l_data_lags[[lag_pos]]) <- paste("V", 1:p, '.lag', lag, '.', sep = "")
 
     } else {
 
       l_data_lags[[lag_pos]] <- data[-c((1:front),((n-end+1):n)), ]
+      l_data_lags[[lag_pos]] <- matrix(l_data_lags[[lag_pos]], ncol = p, nrow = n_var)
       colnames(l_data_lags[[lag_pos]]) <- paste("V", 1:p, '.lag', lag, '.', sep = "")
 
     }
